﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Editor.Commands
{
    public sealed class CommandManager : ICommandManager
    {
        private List<IUndoableCommand> commandHistory;
        private int currentCommnad = -1;
        private IUndoableCommand lastCommand;

        public IUndoableCommand LastCommand
        {
            get { return lastCommand; }
        }

        public IUndoableCommand RedoCommand
        {
            get 
            { 
                if( currentCommnad == commandHistory.Count - 1 ) return null;
    
                return commandHistory[currentCommnad + 1];
            }
        }

        public IUndoableCommand UndoCommand
        {
            get 
            { 
                if( currentCommnad < 0 ) return null;

                return commandHistory[currentCommnad];
            }
        }

        public CommandManager()
        {
            commandHistory = new List<IUndoableCommand>();
            lastCommand = null;
        }

        public bool ExecuteCommand( ICommand command )
        {
            bool executed = command.Execute();

            if( command is IUndoableCommand && executed )
            {
                this.TrimHistory();
                commandHistory.Add( command as IUndoableCommand );
                currentCommnad++;

                lastCommand = command as IUndoableCommand;
            }

            return executed;
        }

        public void Redo()
        {
            if( currentCommnad == commandHistory.Count - 1 ) return;

            IUndoableCommand command = commandHistory[currentCommnad + 1];
            command.Execute();
            currentCommnad++;

            lastCommand = null;
        }

        public void Undo()
        {
            if( currentCommnad < 0 ) return;

            IUndoableCommand command = commandHistory[currentCommnad];
            command.Undo();
            currentCommnad--;

            lastCommand = null;
        }

        private void TrimHistory()
        {
            if( commandHistory.Count == 0 ) return;
            if( currentCommnad == commandHistory.Count - 1 ) return;

            for( int i = commandHistory.Count - 1; i > currentCommnad; i-- )
                commandHistory.RemoveAt(i);
        }
    }
}
