﻿using System;
using System.Collections.Generic;
using System.Text;
using Editor.View;
using Editor.Text;
using Editor.Document;

namespace Editor.Commands.Document.Text
{
    public sealed class InsertText : IUndoableCommand, IDocumentCommand, IAppendableCommand
    {
        private int charIndex;
        private IDocument document;
        private IDocumentRow documentRow;
        private IText inserted;
        private int virtualIndex;

        public IDocument Document
        {
            get { return document; }
        }

        public InsertText( IDocument document, IText text )
        {
            charIndex = -1;
            this.document = document;
            documentRow = null;
            this.inserted = text;
            virtualIndex = -1;
        }

        public void AppendText( IText text )
        {
            if( virtualIndex == -1 ) return;

            this.inserted.InsertText( text.VirtualLength, text );
        }

        public bool CanAppend( IDocumentRow selectionRow, int selectionVirtualIndex )
        {
            return ( object.ReferenceEquals( documentRow, selectionRow ) && 
                ( virtualIndex + inserted.VirtualLength ) == selectionVirtualIndex );
        }

        public bool Execute()
        {
            if( document.Selection.MultiIndex ) return false;

            charIndex = document.Selection.StartIndex;
            documentRow = document[document.Selection.StartRow];

            if( !documentRow.Editable ) return false;

            int startIndex = 0;
            int endIndex = 0;
            
            virtualIndex = documentRow.Text.GetVirtualIndex( charIndex, 
                out startIndex, out endIndex );

            if( charIndex != startIndex ) return false;

            documentRow.Text.InsertText( virtualIndex, inserted );

            int rowIndex = document.IndexOf( documentRow );

            document.Selection.Change( rowIndex, charIndex + inserted.CharacterLength, rowIndex, 
                charIndex + inserted.CharacterLength );

            return true;
        }

        public void  Redo()
        {
            charIndex = -1;
            virtualIndex = -1;

            Execute();
        }

        public void  Undo()
        {
            if( documentRow != null ) return;

            documentRow.Text.Remove( virtualIndex, inserted.VirtualLength );

            int rowIndex = document.IndexOf( documentRow );

            document.Selection.Change( rowIndex, charIndex, rowIndex, charIndex );
        }
    }
}
