﻿using System;
using System.Collections.Generic;
using System.Text;
using Editor.Document;
using Editor.Text;

namespace Editor.Commands.Document.Text
{
    public sealed class StringReplaceText : IUndoableCommand, IDocumentCommand, IAppendableCommand
    {
        private string characters;
        private int charIndex;
        private IDocument document;
        private IDocumentRow documentRow;
        private IText inserted;
        private IText removed;
        private int virtualIndex;

        public IDocument Document 
        {
            get { return document; }
        }

        public StringReplaceText( IDocument document, string characters )
        {
            this.characters = characters;
            charIndex = -1;
            this.document = document;
            documentRow = null;  
            this.inserted = null;
            this.removed = null; 
            virtualIndex = -1;
        }

        public void AppendText( IText text )
        {
            if( virtualIndex == -1 ) return;

            this.inserted.InsertText( text.VirtualLength, text );
        }

        public bool CanAppend( IDocumentRow selectionRow, int selectionVirtualIndex )
        {
            return ( object.ReferenceEquals( documentRow, selectionRow ) && 
                ( virtualIndex + inserted.VirtualLength ) == selectionVirtualIndex );
        }

        public bool Execute()
        {
            if( document.Selection.MultiRow ) return false;

            SSelection selection = document.Selection.GetOrdered();

            charIndex = selection.StartIndex;
            documentRow = document[selection.StartRow];

            if( !documentRow.Editable ) return false;

            int startIndex = 0;
            int endIndex = 0;
            
            virtualIndex = documentRow.Text.GetVirtualIndex( charIndex, 
                out startIndex, out endIndex );

            if( charIndex != startIndex || virtualIndex < 0 ) return false;

            int tempIndex = documentRow.Text.GetVirtualIndex( selection.EndIndex,
                out startIndex, out endIndex );

            if( selection.EndIndex != startIndex || tempIndex < 0 ) return false;

            removed = documentRow.Text.Remove( virtualIndex, tempIndex - virtualIndex );
            inserted = documentRow.Text.InsertString( virtualIndex, characters );

            int rowIndex = document.IndexOf( documentRow );

            document.Selection.Change( rowIndex, charIndex + inserted.CharacterLength, rowIndex,
                charIndex + inserted.CharacterLength );

            return true;
        }

        public void Redo()
        {
            charIndex = -1;
            documentRow = null;
            virtualIndex = -1;

            Execute();
        }

        public void Undo()
        {
            if( virtualIndex == -1 ) return;

            documentRow.Text.Remove( virtualIndex, inserted.VirtualLength );
            documentRow.Text.InsertText( virtualIndex, removed );

            int rowIndex = document.IndexOf( documentRow );

            document.Selection.Change( rowIndex, charIndex + removed.CharacterLength, rowIndex, 
                charIndex + removed.CharacterLength );
        }
    }
}
