﻿using System;
using System.Collections.Generic;
using System.Text;
using Editor.Document.UseCase;
using Editor.Document;
using Editor.Numbering;

namespace Editor.Commands.Document.UseCase
{
    public sealed class AddFlowStep : ICommand, IDocumentCommand
    {
        private UseCaseDocument document;

        public IDocument Document
        {
            get { return document as IDocument; }
        }

        public AddFlowStep( UseCaseDocument document )
        {
            this.document = document;
        }

        public bool Execute()
        {
            if( document.Selection.MultiRow ) return false;

            TemplateRow templateRow = document[document.Selection.StartRow] as TemplateRow;

            if( templateRow is Flow )
            {
                for( int i = 0; i < templateRow.ItemCount; i++ )
                {
                    if( templateRow[i] is StepRow )
                    {
                        templateRow.Insert( i, new StepRow( new DefaultNumbering() ) );
                        
                        document.RecacheRows();

                        int rowIndex = ((IDocument)document).IndexOf(templateRow[i]);

                        document.Selection.Change( rowIndex, 0, rowIndex, 0 );

                        return true;
                    }
                }

                templateRow.Insert( templateRow.ItemCount, new StepRow( new DefaultNumbering() ) );
                        
                document.RecacheRows();

                int documentIndex = ((IDocument)document).IndexOf( templateRow[templateRow.ItemCount-1] );

                document.Selection.Change( documentIndex, 0, documentIndex, 0 );

                return true;
            }
            else if( templateRow is StepRow )
            {
                TemplateRow parentRow = templateRow.Parent;

                int childIndex = parentRow.IndexOf( templateRow );

                parentRow.Insert(childIndex + 1, new StepRow(new DefaultNumbering()));

                document.RecacheRows();

                int rowIndex = ((IDocument)document).IndexOf( parentRow[childIndex + 1] );

                document.Selection.Change( rowIndex, 0, rowIndex, 0 );

                return true;
            }
            else return false;
        }
    }
}
