﻿using System;
using System.Collections.Generic;
using System.Text;
using Editor.Numbering;
using Editor.Text;
using Editor.View;
using System.Drawing;

namespace Editor.Document.UseCase
{
    public sealed class StepRow : TemplateRow, INumbered
    {
        private INumbering numbering;
 
        public string CurrentNumber
        {
            get { return string.Concat( numbering.ToString(), "." ); }
        }

        public override bool Editable
        {
            get { return true; }
        }

        public bool EqualNumberIndent 
        {
            get { return true; }
        }

        public override bool Formattable
        {
            get { return true; }
        }

        public bool LevelChanging
        {
            get { return true; }
        }

        public override bool Removable
        {
            get { return true; }
        }

        public string LongestNumber
        {
            get { return string.Concat( numbering.LongestNumber, "." ); }
        }

        public INumbering Numbering
        {
            get { return numbering; }
        }

        public short NumberingStyle 
        {
            get { return (short)EDefaultStyles.StepNumbering; }
        }

        public event EventHandler NumberingChanged;

        public StepRow( INumbering numbering ) 
        {
            leftIndent = 20;
            this.numbering = numbering;

            numbering.ObjectChanged += OnNumberingChanged;
        }

        public override TemplateRow CreateNewRow()
        {
            return new StepRow( numbering.NextNumber() );
        }

        private void OnNumberingChanged( object sender, EventArgs eventArgs )
        {
            if( NumberingChanged != null ) NumberingChanged( sender, eventArgs );

            ObjectChangedEvent( sender, eventArgs );
        }
    }
}
