﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Editor.Document.UseCase
{
    public class StepNumber : INumber
    {
        private string label;
        private List<byte> numbers;

        public string Label 
        {
            get { return label; }
            set { label = value; }
        }

        public int Level
        {
            get { return numbers.Count; }
        }

        internal List<byte> Numbers
        {
            get { return numbers; }
        }

        public StepNumber() 
        {
            label = string.Empty;
            numbers = new List<byte>();
            
            numbers.Add( 1 );
        }

        public INumber Clone()
        {
            StepNumber stepNumbering = new StepNumber();
            
            stepNumbering.Label = label;
            stepNumbering.Numbers.AddRange( numbers );

            return stepNumbering;
        }

        public void DecreaseNumber()
        {
            if( numbers[numbers.Count - 1] > 1 )
                numbers[numbers.Count - 1]--;
        }

        public void DecreaseLevel()
        {
            if( numbers.Count > 1 )
                numbers.RemoveAt( numbers.Count - 1 );
        }

        public void IncreaseNumber()
        {
            if( numbers[numbers.Count - 1] < byte.MaxValue )
                numbers[numbers.Count - 1]++;
        }

        public void IncreaseLevel()
        {
            if( numbers.Count < 8 )
                numbers.Add( 1 );
        }

        public INumber NextNumber()
        {
            if( numbers[numbers.Count - 1] < byte.MaxValue )
            {
                INumber number = this.Clone();
                number.IncreaseLevel();

                return number;
            }
            return null;
        }

        public INumber PreviousNumber()
        {
            if( numbers[numbers.Count - 1] > 1 )
            {
                INumber number = this.Clone();
                number.DecreaseLevel();

                return number;
            }
            return null;
        }
    }
}
