﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Editor.Text
{
    public sealed class TextIterator : ICloneable<TextIterator>
    {
        private int charIndex;
        private int charOffset;
        private Glyph glyph;
        private int virtualIndex;
        private IText text;

        public char Character
        {
            get { return glyph.GetCharAt( charOffset ); }
        }

        public int CharIndex
        {
            get { return charIndex; }
        }

        public Glyph Glyph
        {
            get { return glyph; }
        }

        public IText Text
        {
            get { return text; }
        }

        public int VirtualIndex
        {
            get { return virtualIndex; }
        }

        public TextIterator( IText text )
        {
            charIndex = -1;
            charOffset = 0;
            glyph = null;
            virtualIndex = 0;
            this.text = text;
        }

        private TextIterator( IText text, int virtualIndex, Glyph glyph, int charIndex, int charOffset )
        {
            this.charIndex = charIndex;
            this.charOffset = charOffset;
            this.glyph = glyph;
            this.virtualIndex = virtualIndex;
            this.text = text;     
        }

        public TextIterator Clone()
        {
            TextIterator textIterator = new TextIterator( text, virtualIndex, glyph,
                charIndex, charOffset );

            return textIterator;
        }

        public bool NextChar()
        {
            if( virtualIndex < text.VirtualLength )
            {
                if( charIndex < 0 )
                {
                    glyph = text[virtualIndex];
                    charIndex = 0;
                }
                else
                {
                    if( charOffset < glyph.CharacterLength - 1 ) charOffset++;
                    else 
                    {
                        virtualIndex++;
                        charOffset = 0;

                        if( virtualIndex < text.VirtualLength ) glyph = text[virtualIndex];
                        else 
                        {
                            glyph = null;
                            return false;
                        }
                    }

                    charIndex++;
                }

                return true;
            }
            else
            {
                glyph = null;
                return false;
            }
        }

        public bool NextGlyph()
        {
            if( virtualIndex < text.VirtualLength )
            {
                virtualIndex++;
                glyph = text[virtualIndex];

                if( virtualIndex > 0 )
                {
                    charIndex -= charOffset;
                    charIndex += text[virtualIndex - 1].CharacterLength;
                }
                else charIndex = 0;

                return true;
            }
            else
            {
                glyph = null;
                return false;
            }
        }

        public void Reset()
        {
            charIndex = -1;
            charOffset = 0;
            glyph = null;
            virtualIndex = 0;
        }

        public bool CanIterate()
        {
            if( virtualIndex < text.VirtualLength ) return true;
            else return false;
        }
    }
}
