﻿using System;
using System.Collections.Generic;
using System.Text;
using Editor.Document.UseCase;
using Editor.Numbering;

namespace Editor.Text.UseCase
{
    public sealed class FlowGlyph : Glyph, IChangeHandler
    {
        private string flowLabel;
        private Flow flow;

        public bool Changing
        {
            get { throw new NotImplementedException(); }
        }

        public override int CharacterLength 
        {
            get { return flowLabel.Length; }
        }

        public Flow Flow
        {
            get { return flow; }
            set 
            {
                if( !object.ReferenceEquals( flow, value ) )
                {
                    flow.ObjectChanged -= OnRowChanged;

                    flow = value;

                    flow.ObjectChanged += OnRowChanged;

                    OnRowChanged( flow, EventArgs.Empty );
                }
            }
        }

        public override bool StyleChanging
        {
            get { return false; }
        }

        public override bool StyleCopying
        {
            get { return false; }
        }

        public override EGlyphType Type
        {
            get { return EGlyphType.Flow; }
        }

        public event EventHandler ObjectChanged;
        public event EventHandler ObjectChanging;

        public FlowGlyph( Flow flow )
        {
            this.flow = flow;
            style = (short)EDefaultStyles.FlowLink;

            flowLabel = CreateLabel();

            flow.ObjectChanged += OnRowChanged;
        }

        public void BeginChange()
        {
            throw new NotImplementedException();
        }

        public override Glyph Clone()
        {
            return new FlowGlyph( flow );
        }

        public void EndChange( bool discardEvent )
        {
            throw new NotImplementedException();
        }

        public Delegate[] GetReferences()
        {
            return ObjectChanged.GetInvocationList();
        }

        public override char GetCharAt( int index )
        {
            return flowLabel[index];
        }

        public override string ToString()
        {
            return flowLabel.ToString();
        }

        private string CreateLabel()
        {
            StringBuilder buffer = new StringBuilder();

            buffer.Append( flow.CurrentNumber );
            buffer.Append(' ');
            buffer.Append( flow.Text.ToString() );

            return buffer.ToString();
        }

        private void OnRowChanged( object sender, EventArgs eventArgs ) 
        {
            flowLabel = CreateLabel();

            if( ObjectChanged != null ) ObjectChanged( sender, eventArgs );
        }
    }
}
