﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Editor.Document;
using Editor.Text;
using Editor.View;
using Editor.Document.UseCase;
using System.Drawing.Drawing2D;
using Editor.Commands.Document.Text;

namespace Editor
{
    public partial class UseCaseEditor : UserControl
    {
        private UseCaseView view;

        public UseCaseView View
        {
            get { return view; }
            set { view = value; }
        }

        public UseCaseEditor()
        {
            SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
            SetStyle(ControlStyles.ResizeRedraw, true);
            SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            SetStyle(ControlStyles.UserPaint, true);
            SetStyle(ControlStyles.CacheText, true);

            AutoScroll = true;

            InitializeComponent();

            BackColor = SystemColors.Window;
        }

        public UseCaseEditor( UseCaseDocument document )
        {
            SetStyle( ControlStyles.DoubleBuffer, true );
            SetStyle( ControlStyles.ResizeRedraw, true );
            SetStyle( ControlStyles.AllPaintingInWmPaint, true );
            SetStyle( ControlStyles.UserPaint, true );   
            AutoScroll = true;

            view = new UseCaseView( this, document );
            view.Prepare();

            InitializeComponent();

            BackColor = SystemColors.Window;

            view.ObjectChanged += OnViewChanged;
        }

        protected override void OnKeyPress( KeyPressEventArgs e )
        {
            base.OnKeyPress( e );

            if( char.IsControl( e.KeyChar ) )
            {
                if( e.KeyChar == '\b' ) OnBackSpace();
            }
            else OnCharacter( e.KeyChar );   
        }

        protected override void OnPaint( PaintEventArgs e )
        {
            base.OnPaint( e );

            Graphics graphics = e.Graphics;
            PaintArguments paintArguments = new PaintArguments( graphics );

            graphics.Clear( BackColor );

            AutoScrollMinSize = new Size( view.Size.Width + 2 * view.Margins, view.Size.Height + 2 * view.Margins );
            graphics.TranslateTransform( this.AutoScrollPosition.X, this.AutoScrollPosition.Y ); 

            view.Draw( paintArguments );
            view.Caret.Draw( paintArguments );
        }

        private void OnBackSpace() 
        {
            if( view.Document.Selection.MultiRow ) return;

            bool executed = false;

            view.Caret.Hide();

            if( view.Document.Selection.MultiIndex ) 
                executed = view.Document.ExecuteCommand( new RemoveText( view.Document ) );
            else executed = view.Document.ExecuteCommand( new RemoveChar( view.Document ) );

            if( executed ) view.PlaceCaret( int.MaxValue );
            else view.Caret.Show();
        }

        private void OnCharacter( char character ) 
        {
            if( view.Document.Selection.MultiRow ) return;

            bool executed = false;

            view.Caret.Hide();

            if( view.Document.Selection.MultiIndex ) 
                executed = view.Document.ExecuteCommand( 
                    new StringReplaceText( view.Document, character.ToString() ) );
            else executed = view.Document.ExecuteCommand( new InsertChar( view.Document, character ) );

            if( executed ) view.PlaceCaret( int.MaxValue );
            else view.Caret.Show();
        }

        private void OnViewChanged( object sender, EventArgs eventArgs )
        {
            this.Invalidate();
        }
    }
}
