﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Editor.Text;

namespace Editor.View
{
    public sealed class NumberSegment : ISegment
    {
        private string number;
        private int indent;
        private IStyle style;
        private IView view;
        private int width;

        public int EndIndex
        {
            get { return -1; }
        }

        public int Length
        {
            get { return number.Length; }
        }

        public int Indent
        {
            get { return indent; }
        }

        public int StartIndex
        {
            get { return -1; }
        }

        public IStyle Style
        {
            get { return style; }
        }

        public int Width
        {
            get { return width; }
        }

        public NumberSegment( IView view, string number, IStyle style, int indent, int width )
        {
            this.number = number;
            this.indent = indent;
            this.style = style;
            this.view = view;
            this.width = width;
        }

        public void Draw( int offset, bool selected, int selectionAscent, int selectionDescent )
        {
            Point location = new Point( indent, offset - style.GetAscent( view ) );

            view.DrawString( number, style.Font, style.TextColor, 
                style.BackColor, location );
        }

        public void Draw( int offset, int selectionStart, int selectionEnd,
            int selectionAscent, int selectionDescent )
        {
            Draw( offset, false, selectionAscent, selectionDescent );
        }

        public int GetSelectionIndex( int x )
        {
            return -1;
        }

        public Size Measure( int index, int length )
        {
            if( length == 0 ) return Size.Empty;

            return view.MeasureString( number.Substring( index, length ),
                style.Font );
        }
    }
}
