﻿using System;
using System.Collections.Generic;
using System.Text;
using Editor.Project;
using System.Windows.Forms;

namespace UseCaseTool.Controls
{
    public class ItemsListBox : ListBox
    {
        protected IProjectItem projectItem;

        public IProjectItem ProjectItem 
        {
            get { return projectItem; }
            set 
            {
                if( projectItem != null ) projectItem.ObjectChanged -= OnProjectItemChanged;

                projectItem = value;

                if( projectItem == null ) Items.Clear();
                else
                {
                    projectItem.ObjectChanged += OnProjectItemChanged;
                    Repopulate();
                }
            }
        }

        public ItemsListBox( IProjectItem projectItem )
        {
            this.projectItem = projectItem;
            if( projectItem != null ) 
            {
                projectItem.ObjectChanged += OnProjectItemChanged;
                Repopulate();
            }
        }

        protected virtual void OnProjectItemChanged( object sender, EventArgs eventArgs )
        {
            Repopulate();
        }

        public void Repopulate()
        {
            if( projectItem == null ) return;

            BeginUpdate();

            Items.Clear();

            for( int i = 0; i < projectItem.ItemCount; i++ )
            {
                IProjectItem subItem = projectItem[i];

                if( ApplyFilter( subItem ) )Items.Add( new ListBoxProjectItem( subItem ) );
            }

            EndUpdate();
        }

        protected virtual bool ApplyFilter( IProjectItem projectItem )
        {
            return true;
        }
    }
}
