﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using UseCaseTool.Forms;
using Editor.Document;
using Editor.Project;
using Editor.Commands.Document.UseCase;
using Editor.Text.UseCase;
using Editor;
using UseCaseTool.Controls;
using Editor.Document.UseCase;

namespace UseCaseTool
{
    public partial class UseCaseTool : Form
    {
        private UseCaseProject project;

        public UseCaseTool()
        {
            InitializeComponent();

            project = null;
        }

        private void projectNewMI_Click(object sender, EventArgs e)
        {
            //if( project != null )
            //{
            //    if( !project.Saved ) 
            //    {
            //        DialogResult dialogResult = MessageBox.Show( "Save project?", "Question", 
            //            MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question );

            //        if( dialogResult == DialogResult.Cancel ) return;
            //        if( dialogResult == DialogResult.Yes ) project.Save( Arguments.Empty );
            //    }

                projectTV.Nodes.Clear();
            //}

            project = new UseCaseProject("New Project");
                
            projectTV.Nodes.Add( project );
        }

        private void projectOpenMI_Click(object sender, EventArgs e)
        {

        }

        private void projectSaveMI_Click(object sender, EventArgs e)
        {
            project.Save( Arguments.Empty );
        }

        private void projectCloseMI_Click(object sender, EventArgs e)
        {
            //if( project != null )
            //{
            //    if( !project.Saved ) 
            //    {
            //        DialogResult dialogResult = MessageBox.Show( "Save project?", "Question", 
            //            MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question );

            //        if( dialogResult == DialogResult.Cancel ) return;
            //        if( dialogResult == DialogResult.Yes ) project.Save( Arguments.Empty );
            //    }

                projectTV.Nodes.Clear();

            //    project = null;
            //}
        }

        private void projectSettingsMI_Click(object sender, EventArgs e)
        {
            if( project == null ) 
                MessageBox.Show( "No project is open", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning );
            else new ProjectSettingsForm( project ).ShowDialog();
        }

        private void dictionaryAddMI_Click(object sender, EventArgs e)
        {
            DefinitionForm definitionForm = null;

            if( project == null ) 
                MessageBox.Show( "No project is open", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning );
            else
            { 
                definitionForm = new DefinitionForm( new DictionaryWord( string.Empty, string.Empty ) );
                DialogResult dialogResult = definitionForm.ShowDialog();

                if( dialogResult == DialogResult.OK ) 
                {
                    project.DictionaryWords.Add( definitionForm.Definition );
                    project.Saved = false;
                }
            }
        }

        private void dictionaryOpenMI_Click(object sender, EventArgs e)
        {
            if( project == null ) 
                MessageBox.Show( "No project is open", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning );
            else new DictionaryForm( project ).ShowDialog();
        }

        private void documentTC_Selected(object sender, TabControlEventArgs e)
        {                        
            DocumentTabPage documentTabPage = documentTC.SelectedTab as DocumentTabPage;

            if( documentTabPage != null ) sectionsIndexSLB.Document = documentTabPage.Document;
            else sectionsIndexSLB.Items.Clear();

        }

        private void documentTC_ControlAdded(object sender, ControlEventArgs e)
        {
            DocumentTabPage documentTabPage = documentTC.SelectedTab as DocumentTabPage;

            if( documentTabPage != null ) sectionsIndexSLB.Document = documentTabPage.Document;
            else sectionsIndexSLB.Items.Clear();
        }

        private void addModelTSB_Click(object sender, EventArgs e)
        {
            if( project == null ) return;

            project.CreateSubItem( new UseCaseModelArguments("New Model") );
        }

        private void removeModelTSB_Click(object sender, EventArgs e)
        {
            if( project == null ) return;

            UseCaseModel model = projectTV.SelectedNode as UseCaseModel;

            if( model != null ) project.Remove( model );
        }

        private void addActorTSB_Click(object sender, EventArgs e)
        {
            if( project == null ) return;

            UseCaseModel model = projectTV.SelectedNode as UseCaseModel;

            if( model == null )
            {
                if( projectTV.SelectedNode == null ) return;

                model = projectTV.SelectedNode.Parent as UseCaseModel;
                if( model == null ) return;
            }

            model.CreateSubItem( new ActorArguments("New Actor") );
        }

        private void removeActorTSB_Click(object sender, EventArgs e)
        {
            if( project == null ) return;

            Actor actor = projectTV.SelectedNode as Actor;

            if( actor != null ) 
            {
                UseCaseModel model = actor.Parent as UseCaseModel;

                if( model != null ) model.Remove( actor );
            }       
        }

        private void addUseCaseTSB_Click(object sender, EventArgs e)
        {
            if( project == null ) return;

            UseCaseModel model = projectTV.SelectedNode as UseCaseModel;

            if( model == null )
            {
                if( projectTV.SelectedNode == null ) return;

                model = projectTV.SelectedNode.Parent as UseCaseModel;
                if( model == null ) return;
            }

            model.CreateSubItem( new UseCaseArguments("New Use Case") );
        }

        private void removeUseCaseTSB_Click(object sender, EventArgs e)
        {
            if( project == null ) return;

            UseCase useCase = projectTV.SelectedNode as UseCase;

            if( useCase != null ) 
            {
                UseCaseModel model = useCase.Parent as UseCaseModel;

                if( model != null ) 
                {
                    DocumentTabPage tabPage = null;

                    for( int i = 0; i < documentTC.TabPages.Count; i++ )
                    {
                        tabPage = documentTC.TabPages[i] as DocumentTabPage;

                        if( tabPage != null && object.ReferenceEquals( tabPage.Document, 
                            useCase.Document ) ) 
                        {
                            documentTC.TabPages.Remove( tabPage );
                        }
                    }

                    model.Remove( useCase );
                }
            }       
        }

        private void projectTV_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if( e.Button != MouseButtons.Left ) return;

            UseCase useCase = e.Node as UseCase;

            if( useCase != null )
            {
                bool documentOpen = false;
                DocumentTabPage tabPage = null;

                for( int i = 0; i < documentTC.TabPages.Count; i++ )
                {
                    tabPage = documentTC.TabPages[i] as DocumentTabPage;

                    if( tabPage != null && object.ReferenceEquals( tabPage.Document, 
                        useCase.Document ) ) 
                    {
                        documentOpen = true;
                        break;
                    }
                }

                if( documentOpen ) documentTC.SelectedTab = tabPage;
                else
                {
                    tabPage = new DocumentTabPage( useCase.Document );

                    documentTC.TabPages.Add( tabPage );
                    documentTC.SelectedTab = tabPage;
                }
            }
        }

        private void projectTV_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if( e.Button != MouseButtons.Right ) return;

            if( e.Node is UseCaseModel )
            {
                ModelForm modelForm = new ModelForm( e.Node as UseCaseModel );
                DialogResult dialogResult = modelForm.ShowDialog();

                if( dialogResult == DialogResult.OK ) project.Saved = false;
            }

            if( e.Node is UseCase )
            {
                UseCaseForm useCaseForm = new UseCaseForm( e.Node as UseCase );
                DialogResult dialogResult = useCaseForm.ShowDialog();

                if( dialogResult == DialogResult.OK ) project.Saved = false;
            }

            if( e.Node is Actor )
            {
                ActorForm actorForm = new ActorForm( e.Node as Actor );
                DialogResult dialogResult = actorForm.ShowDialog();

                if( dialogResult == DialogResult.OK ) project.Saved = false;
            }
        }

        private void insertDictWordTSB_Click(object sender, EventArgs e)
        {
            if( project == null ) return;

            if( documentTC.SelectedIndex < 0 ) return;

            InsertDefinitionForm insertDefinitionForm = new InsertDefinitionForm( project );
            DialogResult dialogResult = insertDefinitionForm.ShowDialog();

            if( dialogResult == DialogResult.OK ) 
            {
                InsertVariationForm insertVariationForm = new InsertVariationForm( insertDefinitionForm.Definition );
                dialogResult = insertVariationForm.ShowDialog();

                if( dialogResult == DialogResult.OK )
                {
                    DocumentTabPage tabPage = documentTC.SelectedTab as DocumentTabPage;

                    if( tabPage != null ) tabPage.Document.ExecuteCommand(
                        new InsertDefinition( tabPage.Document, new DefinitionGlyph( insertDefinitionForm.Definition,
                            insertVariationForm.VariationIndex, insertVariationForm.VariationGroup ) ) );
                }
            }
        }

        private void insertActorTSB_Click(object sender, EventArgs e)
        {
            if( project == null ) return;

            DocumentTabPage tabPage = documentTC.SelectedTab as DocumentTabPage;

            if( tabPage == null ) return;

            InsertActorForm insertActorForm = new InsertActorForm( tabPage.Document.Model );
            DialogResult dialogResult = insertActorForm.ShowDialog();

            if( dialogResult == DialogResult.OK ) 
            {
                InsertVariationForm insertVariationForm = new InsertVariationForm( insertActorForm.Actor );
                dialogResult = insertVariationForm.ShowDialog();

                if( dialogResult == DialogResult.OK )
                {
                    tabPage = documentTC.SelectedTab as DocumentTabPage;

                    if( tabPage != null ) tabPage.Document.ExecuteCommand(
                        new InsertActor( tabPage.Document, new ActorGlyph( insertActorForm.Actor,
                            insertVariationForm.VariationIndex, insertVariationForm.VariationGroup ) ) );
                }
            }
        }

        private void changeVariationTSB_Click(object sender, EventArgs e)
        {

        }

        private void assignActorTSN_Click(object sender, EventArgs e)
        {
            if( project == null ) return;

            DocumentTabPage tabPage = documentTC.SelectedTab as DocumentTabPage;

            if( tabPage == null ) return;

            AssignActorForm assignActorForm = new AssignActorForm( tabPage.Document.Model );
            DialogResult dialogResult = assignActorForm.ShowDialog();

            if( dialogResult == DialogResult.OK ) 
            {
                tabPage = documentTC.SelectedTab as DocumentTabPage;

                tabPage.Document.ExecuteCommand(
                        new AssignActor( tabPage.Document, new ActorGlyph( assignActorForm.Actor,
                            -1, EVariationGroup.Normal ) ) );
            }
        }

        private void deleteActorTSB_Click(object sender, EventArgs e)
        {
            if( project == null ) return;

            DocumentTabPage tabPage = documentTC.SelectedTab as DocumentTabPage;

            if( tabPage == null ) return;

            UseCaseDocument document = tabPage.Document;
            Actors actors = null;

            for( int i = 0; i < document.RowCount; i++ )
            {
                actors = document[i] as Actors;

                if( actors != null )
                {
                    AssignActorForm deleteActorForm = new AssignActorForm( tabPage.Document.Model );
                    DialogResult dialogResult = deleteActorForm.ShowDialog();

                    if( dialogResult == DialogResult.OK ) 
                    {
                        tabPage = documentTC.SelectedTab as DocumentTabPage;

                        tabPage.Document.ExecuteCommand(
                                new AssignActor( tabPage.Document, new ActorGlyph( deleteActorForm.Actor,
                                    -1, EVariationGroup.Normal ) ) );
                    }

                    break;
                }
            }

        }

        private void addMainFlowTSB_Click(object sender, EventArgs e)
        {
            if( project == null ) return;

            DocumentTabPage tabPage = documentTC.SelectedTab as DocumentTabPage;

            if( tabPage == null ) return;

            UseCaseDocument document = tabPage.Document;

            document.ExecuteCommand( new AddMainFlow( document ) );
        }

        private void addSubflowTSB_Click(object sender, EventArgs e)
        {
            if( project == null ) return;

            DocumentTabPage tabPage = documentTC.SelectedTab as DocumentTabPage;

            if( tabPage == null ) return;

            UseCaseDocument document = tabPage.Document;

            document.ExecuteCommand( new AddSubflow( document ) );
        }

        private void addAlternativeFlowTSB_Click(object sender, EventArgs e)
        {
            if( project == null ) return;

            DocumentTabPage tabPage = documentTC.SelectedTab as DocumentTabPage;

            if( tabPage == null ) return;

            UseCaseDocument document = tabPage.Document;

            document.ExecuteCommand( new AddAlternativeFlow( document) );
        }

        private void addFlowStepTSB_Click(object sender, EventArgs e)
        {
            if( project == null ) return;

            DocumentTabPage tabPage = documentTC.SelectedTab as DocumentTabPage;

            if( tabPage == null ) return;

            UseCaseDocument document = tabPage.Document;

            document.ExecuteCommand( new AddFlowStep( document ) );
        }
    }
}
